#!/usr/bin/env python3
"""
VPN Manager
VPN connection management and monitoring tool.
Parameters:
    - config: Path to OpenVPN config file
    - auth: Path to authentication file
    - monitor: Enable connection monitoring
Requires: openvpn package and admin/root privileges
"""
import subprocess
import argparse
import logging
from datetime import datetime
import json
import threading
import queue
import os
import signal
import time
import re
import psutil
import netifaces
import requests

class VPNManager:
    def __init__(self, config_file, auth_file=None, monitor=False):
        self.config_file = config_file
        self.auth_file = auth_file
        self.monitor = monitor
        self.vpn_process = None
        self.stop_event = threading.Event()
        self.status_queue = queue.Queue()
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def get_original_ip(self):
        """Get original public IP address"""
        try:
            response = requests.get('https://api.ipify.org?format=json')
            return response.json()['ip']
        except Exception as e:
            self.logger.error(f"Error getting original IP: {str(e)}")
            return None

    def get_vpn_ip(self):
        """Get current VPN IP address"""
        try:
            for interface in netifaces.interfaces():
                addrs = netifaces.ifaddresses(interface)
                if netifaces.AF_INET in addrs:
                    for addr in addrs[netifaces.AF_INET]:
                        if 'addr' in addr and not addr['addr'].startswith('127.'):
                            return addr['addr']
        except Exception as e:
            self.logger.error(f"Error getting VPN IP: {str(e)}")
        return None

    def parse_openvpn_log(self, log_line):
        """Parse OpenVPN log output"""
        status = {
            'timestamp': datetime.now().isoformat(),
            'type': 'info',
            'message': log_line
        }

        # Connection success
        if 'Initialization Sequence Completed' in log_line:
            status.update({
                'type': 'success',
                'message': 'VPN connection established'
            })

        # Connection error
        elif any(err in log_line for err in ['ERROR', 'FATAL']):
            status.update({
                'type': 'error',
                'message': log_line
            })

        # Authentication
        elif 'AUTH_FAILED' in log_line:
            status.update({
                'type': 'error',
                'message': 'Authentication failed'
            })

        # TLS error
        elif 'TLS Error' in log_line:
            status.update({
                'type': 'error',
                'message': 'TLS handshake failed'
            })

        return status

    def monitor_connection(self):
        """Monitor VPN connection status"""
        original_ip = self.get_original_ip()
        self.logger.info(f"Original Public IP: {original_ip}")
        
        while not self.stop_event.is_set():
            try:
                # Check if VPN process is still running
                if self.vpn_process and self.vpn_process.poll() is not None:
                    self.status_queue.put({
                        'timestamp': datetime.now().isoformat(),
                        'type': 'error',
                        'message': 'VPN process terminated unexpectedly'
                    })
                    break

                # Get current IP and compare
                current_ip = self.get_vpn_ip()
                if current_ip and current_ip != original_ip:
                    self.status_queue.put({
                        'timestamp': datetime.now().isoformat(),
                        'type': 'info',
                        'message': f'Current VPN IP: {current_ip}'
                    })

                # Check network interface statistics
                for interface in netifaces.interfaces():
                    if interface.startswith('tun') or interface.startswith('tap'):
                        addrs = netifaces.ifaddresses(interface)
                        if netifaces.AF_INET in addrs:
                            self.status_queue.put({
                                'timestamp': datetime.now().isoformat(),
                                'type': 'info',
                                'message': f'VPN Interface {interface}: '
                                         f'{addrs[netifaces.AF_INET]}'
                            })

                time.sleep(5)
                
            except Exception as e:
                self.logger.error(f"Error monitoring connection: {str(e)}")
                time.sleep(5)

    def start_vpn(self):
        """Start OpenVPN connection"""
        try:
            cmd = ['openvpn', '--config', self.config_file]
            
            if self.auth_file:
                cmd.extend(['--auth-user-pass', self.auth_file])
            
            self.vpn_process = subprocess.Popen(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True
            )
            
            self.logger.info("Starting VPN connection...")
            
            # Start monitoring thread if enabled
            if self.monitor:
                monitor_thread = threading.Thread(target=self.monitor_connection)
                monitor_thread.daemon = True
                monitor_thread.start()
            
            # Process OpenVPN output
            while True:
                line = self.vpn_process.stdout.readline()
                if not line:
                    break
                    
                status = self.parse_openvpn_log(line.strip())
                self.status_queue.put(status)
                
                if status['type'] == 'error':
                    self.logger.error(status['message'])
                elif status['type'] == 'success':
                    self.logger.info(status['message'])
                else:
                    self.logger.debug(status['message'])
            
        except Exception as e:
            self.logger.error(f"Error starting VPN: {str(e)}")
        finally:
            self.stop_vpn()

    def stop_vpn(self):
        """Stop OpenVPN connection"""
        try:
            if self.vpn_process:
                self.stop_event.set()
                self.vpn_process.terminate()
                self.vpn_process.wait(timeout=5)
                self.logger.info("VPN connection terminated")
        except Exception as e:
            self.logger.error(f"Error stopping VPN: {str(e)}")
            if self.vpn_process:
                self.vpn_process.kill()

    def save_logs(self, output_file):
        logs = []
        while not self.status_queue.empty():
            logs.append(self.status_queue.get())

        log_data = {
            'timestamp': datetime.now().isoformat(),
            'config_file': self.config_file,
            'original_ip': self.get_original_ip(),
            'current_ip': self.get_vpn_ip(),
            'logs': logs
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(log_data, f, indent=4)
            self.logger.info(f"Logs saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving logs: {str(e)}")

    def print_status(self):
        """Print current VPN status"""
        current_ip = self.get_vpn_ip()
        
        self.logger.info("\nVPN Status:")
        if self.vpn_process and self.vpn_process.poll() is None:
            self.logger.info("Status: Connected")
            self.logger.info(f"Current IP: {current_ip}")
            
            # Show interface statistics
            for interface in netifaces.interfaces():
                if interface.startswith('tun') or interface.startswith('tap'):
                    self.logger.info(f"\nVPN Interface: {interface}")
                    addrs = netifaces.ifaddresses(interface)
                    for addr_family in addrs:
                        self.logger.info(f"Addresses: {addrs[addr_family]}")
        else:
            self.logger.info("Status: Disconnected")

    def start(self, output_file=None):
        try:
            self.start_vpn()
        except KeyboardInterrupt:
            self.logger.info("Received interrupt signal")
        finally:
            self.stop_vpn()
            if output_file:
                self.save_logs(output_file)
            self.print_status()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="VPN Manager")
    parser.add_argument("config", help="Path to OpenVPN config file")
    parser.add_argument("--auth", help="Path to authentication file")
    parser.add_argument("--monitor", action="store_true",
                      help="Enable connection monitoring")
    parser.add_argument("--output", help="Output file for connection logs")
    args = parser.parse_args()

    manager = VPNManager(args.config, args.auth, args.monitor)
    manager.start(args.output)
